@@ -15,9 +15,9 @@ class Accountant:
     """
     DEFAULT_CASH_BOOK_ID = {'name': 'cash_book', 'id': '1'}
 
-    def __init__(self, bank_account):
+    def __init__(self, bank_account, ledger_host, ledger_read_host=None):
         self.bank_account = bank_account
-        self.ledger = LedgerClient(ledger_host="http://internal-prod-internal-loadbalancer-874786328.ap-southeast-1.elb.amazonaws.com/")
+        self.ledger = LedgerClient(ledger_endpoint=ledger_host, ledger_read_only_endpoint=ledger_read_host)
 
         log.info("---------Initialising-------------")
         log.info(f"Bank Account Number:  {self.bank_account['account_id']}")
@@ -32,17 +32,17 @@ class Accountant:
         book_balance = []
         for currency, amount in blocked_balance_book.items():
             if currency in main_balance_book:
-                main_balance = float(main_balance_book[currency])
+                main_balance = main_balance_book[currency]
             else:
-                main_balance = float(0)
+                main_balance = "0"
             if currency in blocked_balance_deposit_book:
-                blocked_balance_deposit = float(blocked_balance_deposit_book[currency])
+                blocked_balance_deposit = blocked_balance_deposit_book[currency]
             else:
-                blocked_balance_deposit = float(0)
+                blocked_balance_deposit = "0"
             if currency in blocked_balance_withdraw_book:
-                blocked_balance_withdraw = float(blocked_balance_withdraw_book[currency])
+                blocked_balance_withdraw = blocked_balance_withdraw_book[currency]
             else:
-                blocked_balance_withdraw = float(0)
+                blocked_balance_withdraw = "0"
             balance = {
                 'currency': currency,
                 'blocked_balance_deposit': blocked_balance_deposit,
@@ -52,82 +52,82 @@ class Accountant:
             book_balance.append(dict(balance))
         return book_balance
 
-    def get_book_balances(self, currency: str, metadata_filter: dict = {}):
+    def get_book_balances(self, currency: str, metadata_filter: dict = {}, timestamp: int = None):
         log.info("---------Balance----------")
         blocked_deposit_balance = self.ledger.get_book_balances(
             book_id=self.bank_account['ledger_books']['blocked_balance']['id'],
-            asset_id=currency, metadata_filter={**{'operation': 'DEPOSIT'}, **metadata_filter})
+            asset_id=currency, metadata_filter={**{'operation': 'DEPOSIT'}, **metadata_filter}, timestamp=timestamp)
         blocked_withdraw_balance = self.ledger.get_book_balances(
             book_id=self.bank_account['ledger_books']['blocked_balance']['id'],
-            asset_id=currency, metadata_filter={**{'operation': 'WITHDRAW'}, **metadata_filter})
+            asset_id=currency, metadata_filter={**{'operation': 'WITHDRAW'}, **metadata_filter}, timestamp=timestamp)
         main_balance = self.ledger.get_book_balances(book_id=self.bank_account['ledger_books']['main_balance']['id'],
-                                                     asset_id=currency, metadata_filter={**metadata_filter})
+                                                     asset_id=currency, metadata_filter={**metadata_filter}, timestamp=timestamp)
         return {'currency': currency,
-                'blocked_balance_deposit': float(blocked_deposit_balance.get(currency, 0)),
-                'blocked_balance_withdraw': float(blocked_withdraw_balance.get(currency, 0)),
-                'main_balance': float(main_balance.get(currency, 0))}
+                'blocked_balance_deposit': blocked_deposit_balance.get(currency, "0"),
+                'blocked_balance_withdraw': blocked_withdraw_balance.get(currency, "0"),
+                'main_balance': main_balance.get(currency, "0")}
 
     def withdraw_block_amount(self, value: str, currency: str, memo: str, metadata: dict = {}):
         log.info("---------Block_Amount_Withdraw----------")
-        operation = self.ledger.post_transfer_sync(
+        operation = self.ledger.post_transfer(
             from_book_id=self.bank_account['ledger_books']['main_balance']['id'],
             to_book_id=self.bank_account['ledger_books']['blocked_balance']['id'], asset_id=currency, value=value,
             memo=memo, metadata={**{"operation": "WITHDRAW"}, **metadata})
         log.info(operation)
         return operation
 
-    def deposit_block_amount(self, value: str, currency: str, memo: str):
+    def deposit_block_amount(self, value: str, currency: str, memo: str, metadata: dict = {}):
         log.info("---------Block_Amount_Deposit----------")
-        operation = self.ledger.post_transfer_sync(
+        operation = self.ledger.post_transfer(
             from_book_id=self.DEFAULT_CASH_BOOK_ID['id'],
             to_book_id=self.bank_account['ledger_books']['blocked_balance']['id'], asset_id=currency, value=value,
             memo=memo,
-            metadata={'operation': 'DEPOSIT'})
+            metadata={**{'operation': 'DEPOSIT'}, **metadata})
         log.info(operation)
         return operation
 
-    def withdraw_unblock_amount(self, value: str, currency: str, memo: str):
+    def withdraw_unblock_amount(self, value: str, currency: str, memo: str, metadata: dict = {}):
         log.info("---------Unblock_Amount_Withdraw----------")
-        operation = self.ledger.post_transfer_sync(
+        operation = self.ledger.post_transfer(
             from_book_id=self.bank_account['ledger_books']['blocked_balance']['id'],
             to_book_id=self.bank_account['ledger_books']['main_balance']['id'], asset_id=currency, value=value,
             memo=memo,
-            metadata={'operation': 'WITHDRAW'})
+            metadata={**{'operation': 'WITHDRAW'}, **metadata})
         log.info(operation)
         return operation
 
-    def deposit_unblock_amount(self, value: str, currency: str, memo: str):
+    def deposit_unblock_amount(self, value: str, currency: str, memo: str, metadata: dict = {}):
         log.info("---------Unblock_Amount_Deposit----------")
-        operation = self.ledger.post_transfer_sync(
+        operation = self.ledger.post_transfer(
             from_book_id=self.bank_account['ledger_books']['blocked_balance']['id'],
             to_book_id=self.DEFAULT_CASH_BOOK_ID['id'], asset_id=currency, value=value, memo=memo,
-            metadata={'operation': 'DEPOSIT'})
+            metadata={**{'operation': 'DEPOSIT'}, **metadata})
         log.info(operation)
         return operation
 
-    def withdraw_amount(self, value: str, currency: str, memo: str):
+    def withdraw_amount(self, value: str, currency: str, memo: str, metadata: dict = {}):
         log.info("---------Withdraw_amount----------")
-        operation = self.ledger.post_transfer_sync(
+        operation = self.ledger.post_transfer(
             from_book_id=self.bank_account['ledger_books']['blocked_balance']['id'],
             to_book_id=self.DEFAULT_CASH_BOOK_ID['id'],
-            asset_id=currency, value=value, memo=memo, metadata={'operation': 'WITHDRAW'})
+            asset_id=currency, value=value, memo=memo, metadata={**{'operation': 'WITHDRAW'}, **metadata})
         log.info(operation)
         return operation
 
-    def deposit_amount(self, value: str, currency: str, memo: str):
+    def deposit_amount(self, value: str, currency: str, memo: str, metadata: dict = {}):
         log.info("---------Deposit_amount----------")
-        operation = self.ledger.post_transfer_sync(
+        operation = self.ledger.post_transfer(
             from_book_id=self.bank_account['ledger_books']['blocked_balance']['id'],
             to_book_id=self.bank_account['ledger_books'][
                 'main_balance']['id'], asset_id=currency, value=value,
-            memo=memo, metadata={'operation': 'DEPOSIT'})
+            memo=memo, metadata={**{'operation': 'DEPOSIT'}, **metadata})
         log.info(operation)
         return operation
 
-    def block_trade_amount(self, from_currency: str, to_currency: str, from_amount: str, to_amount: str, memo: str):
+    def block_trade_amount(self, from_currency: str, to_currency: str, from_amount: str, to_amount: str, memo: str, metadata: dict = {}):
         log.info("---------Block_Trade_Amount----------")
-        withdraw_block_amount = self.withdraw_block_amount(value=from_amount, currency=from_currency, memo=memo)
-        deposit_block_amount = self.deposit_block_amount(value=to_amount, currency=to_currency, memo=memo)
+        withdraw_block_amount = self.withdraw_block_amount(value=from_amount, currency=from_currency, memo=memo, metadata=metadata)
+        deposit_block_amount = self.deposit_block_amount(value=to_amount, currency=to_currency, memo=memo, metadata=metadata)
         operation = {
             'withdraw_block_amount': withdraw_block_amount,
             'deposit_block_amount': deposit_block_amount
@@ -135,10 +135,10 @@ class Accountant:
         log.info(operation)
         return operation
 
-    def trade_amount(self, from_currency: str, to_currency: str, from_amount: str, to_amount: str, memo: str):
+    def trade_amount(self, from_currency: str, to_currency: str, from_amount: str, to_amount: str, memo: str, metadata: dict = {}):
         log.info("---------Trade_Amount----------")
-        withdraw_amount = self.withdraw_amount(value=from_amount, currency=from_currency, memo=memo)
-        deposit_amount = self.deposit_amount(value=to_amount, currency=to_currency, memo=memo)
+        withdraw_amount = self.withdraw_amount(value=from_amount, currency=from_currency, memo=memo, metadata=metadata)
+        deposit_amount = self.deposit_amount(value=to_amount, currency=to_currency, memo=memo, metadata=metadata)
         operation = {
             'withdraw_block_amount': withdraw_amount,
             'deposit_block_amount': deposit_amount
@@ -146,10 +146,24 @@ class Accountant:
         log.info(operation)
         return operation
 
+    def freeze_books(self):
+        log.info("---------Freezing Ledger Books----------")
+        blocked_book = self.ledger.freeze_book(self.bank_account['ledger_books']['blocked_balance']['id'])
+        main_book = self.ledger.freeze_book(self.bank_account['ledger_books']['main_balance']['id'])
+        log.info(blocked_book, main_book)
+        return main_book, blocked_book
+
+    def unfreeze_books(self):
+        log.info("---------Unfreezing Ledger Books----------")
+        blocked_book = self.ledger.unfreeze_book(self.bank_account['ledger_books']['blocked_balance']['id'])
+        main_book = self.ledger.unfreeze_book(self.bank_account['ledger_books']['main_balance']['id'])
+        log.info(blocked_book, main_book)
+        return main_book, blocked_book
+
     @staticmethod
-    def create_book(account_id: str):
+    def create_book(account_id: str, ledger_host):
         log.info("---------New_ledger-------------")
-        ledger = LedgerClient(ledger_host="http://internal-prod-internal-loadbalancer-874786328.ap-southeast-1.elb.amazonaws.com/")
+        ledger = LedgerClient(ledger_endpoint=ledger_host)
         block_book_id = ledger.create_book(name=f"{account_id}_block_book", min_balance="0")
         main_book_id = ledger.create_book(name=f"{account_id}_main_book", min_balance="0")
         new_books = {
@@ -159,4 +173,4 @@ class Accountant:
                                 'id': block_book_id['id']}
         }
         log.info(new_books)
-        return new_books
+        return new_books
\ No newline at end of file
